package com.atlassian.jira.plugin.ext.bazaar;

import com.opensymphony.module.propertyset.PropertySet;

/**
 * 
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 *
 */
public interface BzrProperties {
	String getRoot();

	String getDisplayName();

	Boolean getRevisionIndexing();

	Integer getRevisionCacheSize();

	String getWebLinkType();

	String getChangesetFormat();

	String getFileAddedFormat();

	String getViewFormat();

	String getFileModifiedFormat();

	String getFileReplacedFormat();

	String getFileDeletedFormat();

	String getXmlRpcPort();

	String getBzrExecutable();

	static class Util {
		static PropertySet fillPropertySet(BzrProperties properties,
				PropertySet propertySet) {
			propertySet.setString(
					MultipleBazaarRepositoryManager.BZR_BRANCH_ROOT_KEY,
					properties.getRoot());
			propertySet.setString(
					MultipleBazaarRepositoryManager.BZR_REPOSITORY_NAME,
					properties.getDisplayName() != null ? properties
							.getDisplayName() : properties.getRoot());
			propertySet.setBoolean(
					MultipleBazaarRepositoryManager.BZR_REVISION_INDEXING_KEY,
					properties.getRevisionIndexing().booleanValue());
			propertySet
					.setInt(
							MultipleBazaarRepositoryManager.BZR_REVISION_CACHE_SIZE_KEY,
							properties.getRevisionCacheSize().intValue());
			propertySet.setString(
					MultipleBazaarRepositoryManager.BZR_LINKFORMAT_TYPE,
					properties.getWebLinkType());
			propertySet.setString(
					MultipleBazaarRepositoryManager.BZR_LINKFORMAT_PATH_KEY,
					properties.getViewFormat()); /* SVN-190 */
			propertySet.setString(
					MultipleBazaarRepositoryManager.BZR_LINKFORMAT_CHANGESET,
					properties.getChangesetFormat());
			propertySet.setString(
					MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_ADDED,
					properties.getFileAddedFormat());
			propertySet
					.setString(
							MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_MODIFIED,
							properties.getFileModifiedFormat());
			propertySet
					.setString(
							MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_REPLACED,
							properties.getFileReplacedFormat());
			propertySet
					.setString(
							MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_DELETED,
							properties.getFileDeletedFormat());
			propertySet.setString(
					MultipleBazaarRepositoryManager.BZR_EXECUTABLE, properties
							.getBzrExecutable());
			propertySet.setString(
					MultipleBazaarRepositoryManager.BZR_XMLRPC_PORT, properties
							.getXmlRpcPort());
			return propertySet;
		}
	}
}
