package com.atlassian.jira.plugin.ext.bazaar;

import java.util.Collection;

import org.vcs.bazaar.client.BazaarPreference;

import com.atlassian.jira.extension.Startable;
import com.atlassian.jira.plugin.ext.bazaar.revisions.RevisionIndexer;

/**
 * Main component of the Bazaar plugin.
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 */
public interface MultipleBazaarRepositoryManager extends Startable {
	String BZR_BRANCH_ROOT_KEY = "bzr.root";
	String BZR_REPOSITORY_NAME = "bzr.display.name";
	String BZR_USERNAME_KEY = "bzr.username";
	String BZR_PASSWORD_KEY = "bzr.password";
	String BZR_PRIVATE_KEY_FILE = "bzr.privatekeyfile";
	String BZR_REVISION_INDEXING_KEY = "revision.indexing";
	String BZR_REVISION_CACHE_SIZE_KEY = "revision.cache.size";
	String BZR_XMLRPC_PORT = "bzr.xmlRpcPort";// BazaarPreference.BZR_XMLRPC_PORT.name();
	String BZR_EXECUTABLE = "bzr."
			+ BazaarPreference.EXECUTABLE.name().toLowerCase();

	String BZR_LINKFORMAT_TYPE = "linkformat.type";
	String BZR_LINKFORMAT_CHANGESET = "linkformat.changeset";
	String BZR_LINKFORMAT_FILE_ADDED = "linkformat.file.added";
	String BZR_LINKFORMAT_FILE_MODIFIED = "linkformat.file.modified";
	String BZR_LINKFORMAT_FILE_REPLACED = "linkformat.file.replaced";
	String BZR_LINKFORMAT_FILE_DELETED = "linkformat.file.deleted";

	String BZR_LINKFORMAT_PATH_KEY = "linkformat.copyfrom";

	String BZR_LOG_MESSAGE_CACHE_SIZE_KEY = "logmessage.cache.size";

	boolean isIndexingRevisions();

	RevisionIndexer getRevisionIndexer();

	/**
	 * Returns a Collection of BazaarManager instances, one for each repository.
	 * 
	 * @return the managers.
	 */
	Collection<BazaarManager> getRepositoryList();

	BazaarManager getRepository(long repoId);

	BazaarManager createRepository(BzrProperties props);

	BazaarManager updateRepository(long repoId, BzrProperties props);

	void removeRepository(long repoId);
}
