package com.atlassian.jira.plugin.ext.bazaar.action;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import com.atlassian.jira.plugin.ext.bazaar.MultipleBazaarRepositoryManager;
import com.atlassian.jira.plugin.ext.bazaar.WebLinkType;
import com.atlassian.jira.security.Permissions;
import com.atlassian.jira.web.action.JiraWebActionSupport;

/**
 * Base class for the Bazaar plugins actions.
 */
public class BazaarActionSupport extends JiraWebActionSupport {

	private static final long serialVersionUID = 1807347033872927522L;

	private MultipleBazaarRepositoryManager multipleRepoManager;
	private List<WebLinkType> webLinkTypes;

	public BazaarActionSupport(MultipleBazaarRepositoryManager manager) {
		this.multipleRepoManager = manager;
	}

	protected MultipleBazaarRepositoryManager getMultipleRepoManager() {
		return multipleRepoManager;
	}

	public boolean hasPermissions() {
		return isHasPermission(Permissions.ADMINISTER);
	}

	public String doDefault() {
		if (!hasPermissions()) {
			return PERMISSION_VIOLATION_RESULT;
		}

		return INPUT;
	}

	public List<WebLinkType> getWebLinkTypes() throws IOException {
		if (webLinkTypes == null) {
			webLinkTypes = new ArrayList<WebLinkType>();
			Properties properties = new Properties();
			properties.load(getClass().getResourceAsStream(
					"/weblinktypes.properties"));

			String[] types = properties.getProperty("types", "").split(" ");
			for (int i = 0; i < types.length; i++) {
				webLinkTypes.add(new WebLinkType(types[i], properties
						.getProperty(types[i] + ".name", types[i]), properties
						.getProperty(types[i] + ".view"), properties
						.getProperty(types[i] + ".changeset"), properties
						.getProperty(types[i] + ".file.added"), properties
						.getProperty(types[i] + ".file.modified"), properties
						.getProperty(types[i] + ".file.replaced"), properties
						.getProperty(types[i] + ".file.deleted")));
			}
		}
		return webLinkTypes;
	}
}
