package com.atlassian.jira.plugin.ext.bazaar.action;

import com.atlassian.jira.plugin.ext.bazaar.BazaarManager;
import com.atlassian.jira.plugin.ext.bazaar.MultipleBazaarRepositoryManager;

public class UpdateBazaarRepositoryAction extends AddBazaarRepositoryAction {

	private static final long serialVersionUID = 6600153100270092817L;

	private long repoId = -1;

	public UpdateBazaarRepositoryAction(
			MultipleBazaarRepositoryManager multipleRepoManager) {
		super(multipleRepoManager);
	}

	public String doDefault() {
		if (ERROR.equals(super.doDefault()))
			return ERROR;

		if (!hasPermissions()) {
			return PERMISSION_VIOLATION_RESULT;
		}

		if (repoId == -1) {
			addErrorMessage(getText("bazaar.repository.id.missing"));
			return ERROR;
		}

		// Retrieve the cvs repository
		final BazaarManager repository = getMultipleRepoManager()
				.getRepository(repoId);
		if (repository == null) {
			addErrorMessage(getText("bazaar.repository.does.not.exist", Long
					.toString(repoId)));
			return ERROR;
		}

		this.setDisplayName(repository.getDisplayName());
		this.setRoot(repository.getRoot());
		if (repository.getViewLinkFormat() != null) {
			this.setWebLinkType(repository.getViewLinkFormat().getType());
			this.setChangesetFormat(repository.getViewLinkFormat()
					.getChangesetFormat());
			this.setViewFormat(repository.getViewLinkFormat().getViewFormat());
			this.setFileAddedFormat(repository.getViewLinkFormat()
					.getFileAddedFormat());
			this.setFileDeletedFormat(repository.getViewLinkFormat()
					.getFileDeletedFormat());
			this.setFileModifiedFormat(repository.getViewLinkFormat()
					.getFileModifiedFormat());
			this.setFileReplacedFormat(repository.getViewLinkFormat()
					.getFileReplacedFormat());
		}
		this.setUsername(repository.getUsername());
		this.setPassword(repository.getPassword());
		this.setPrivateKeyFile(repository.getPrivateKeyFile());
		this.setRevisionCacheSize(new Integer(repository
				.getRevisioningCacheSize()));
		this.setRevisionIndexing(new Boolean(repository.isRevisionIndexing()));
		// this.setXmlRpcPort(new String(repository.getXmlRpcPort()));
		// this.setBzrExecutable(new String(repository.getBzrExecutable()));

		return INPUT;
	}

	public String doExecute() {
		if (!hasPermissions()) {
			addErrorMessage(getText("bazaar.admin.privilege.required"));
			return ERROR;
		}

		if (repoId == -1) {
			return getRedirect("ViewBazaarRepositories.jspa");
		}

		BazaarManager bazaarManager = getMultipleRepoManager()
				.updateRepository(repoId, this);
		if (!bazaarManager.isActive()) {
			repoId = bazaarManager.getId();
			addErrorMessage(bazaarManager.getInactiveMessage());
			addErrorMessage(getText("admin.errors.occured.when.updating"));
			return ERROR;
		}
		return getRedirect("ViewBazaarRepositories.jspa");
	}

	public long getRepoId() {
		return repoId;
	}

	public void setRepoId(long repoId) {
		this.repoId = repoId;
	}

}
