package com.atlassian.jira.plugin.ext.bazaar.issuetabpanels.changes;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.vcs.bazaar.client.IBazaarLogMessage;

import com.atlassian.jira.issue.Issue;
import com.atlassian.jira.issue.action.IssueActionComparator;
import com.atlassian.jira.issue.tabpanels.GenericMessageAction;
import com.atlassian.jira.plugin.ext.bazaar.MultipleBazaarRepositoryManager;
import com.atlassian.jira.plugin.issuetabpanel.AbstractIssueTabPanel;
import com.atlassian.jira.security.PermissionManager;
import com.atlassian.jira.security.Permissions;
import com.atlassian.jira.util.EasyList;
import com.opensymphony.user.User;
import java.util.LinkedList;

public class BazaarRevisionsTabPanel extends AbstractIssueTabPanel {
	private static Logger log = Logger.getLogger(BazaarRevisionsTabPanel.class);

	protected final MultipleBazaarRepositoryManager multipleBazaarRepositoryManager;
	private PermissionManager permissionManager;

	public BazaarRevisionsTabPanel(
			MultipleBazaarRepositoryManager multipleBazaarRepositoryManager,
			PermissionManager permissionManager) {
		this.multipleBazaarRepositoryManager = multipleBazaarRepositoryManager;
		this.permissionManager = permissionManager;
	}

	public List<BazaarRevisionAction> getActions(Issue issue, User remoteUser) {
		try {
			Map<Long, List<IBazaarLogMessage>> logEntries = multipleBazaarRepositoryManager
					.getRevisionIndexer().getLogEntriesByRepository(issue);


			// TODO: Do we need to pass an error message and if so, how?
			// Return an empty list of actions if we have no entries
			if (logEntries == null || logEntries.size() == 0) {
			    return new LinkedList<BazaarRevisionAction>();
			} else {
				List<BazaarRevisionAction> actions = new ArrayList<BazaarRevisionAction>(
						logEntries.size());
				for (Map.Entry<Long, List<IBazaarLogMessage>> entry : logEntries
						.entrySet()) {
					long repoId = ((Long) entry.getKey()).longValue();
					for (IBazaarLogMessage logEntry : entry.getValue()) {
						actions
								.add(createBazaarRevisionAction(repoId,
										logEntry));
					}
				}
				Collections.sort(actions, IssueActionComparator.COMPARATOR);
				return actions;
			}
		} catch (Throwable t) {
			log.error("Error retrieving actions for : " + issue.getKey(), t);
		}
		return Collections.EMPTY_LIST;
	}

	protected BazaarRevisionAction createBazaarRevisionAction(long repoId,
			IBazaarLogMessage logEntry) {
		return new BazaarRevisionAction(logEntry,
				multipleBazaarRepositoryManager, descriptor, repoId);
	}

	protected String getText(String key) {
		return descriptor.getI18nBean().getText(key);
	}

	public boolean showPanel(Issue issue, User remoteUser) {
		return multipleBazaarRepositoryManager.isIndexingRevisions()
				&& permissionManager.hasPermission(
						Permissions.VIEW_VERSION_CONTROL, issue, remoteUser);
	}
}