package com.atlassian.jira.plugin.ext.bazaar;

import java.util.HashMap;

import junit.framework.TestCase;

import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.module.propertyset.memory.MemoryPropertySet;

public class TestBazaarManagerImpl extends TestCase
{

    private BazaarManager bzrManager;

    private PropertySet propertySet;

    private BazaarProperties bzrProperties;

    protected void setUp() throws Exception {
        super.setUp();
        propertySet = new MemoryPropertySet();
        propertySet.init(new HashMap(), new HashMap());

        bzrProperties = new BazaarProperties();
    }

    public void testIndexingOff() {

        propertySet.setString(MultipleBazaarRepositoryManager.BZR_BRANCH_ROOT_KEY, "/root/repo");
        propertySet.setString(MultipleBazaarRepositoryManager.BZR_REPOSITORY_NAME, "name");
        propertySet.setString(MultipleBazaarRepositoryManager.BZR_USERNAME_KEY, "name");
        propertySet.setString(MultipleBazaarRepositoryManager.BZR_PASSWORD_KEY, "password");
        propertySet.setBoolean(MultipleBazaarRepositoryManager.BZR_REVISION_INDEXING_KEY, false);


        bzrManager = new BazaarManagerImpl(1l, propertySet) {
            public ViewLinkFormat getViewLinkFormat()
            {
                return null;
            }
        };
        // existing behaviour is that we will have a subversion manager with no repo, assert we have the mgr
        assertNotNull(bzrManager);
        assertFalse(bzrManager.isRevisionIndexing());
    }

    public void testUpdateWeblinksConfiguration() {
        ViewLinkFormat viewLinkFormat;

        bzrProperties.setRevisionIndexing(Boolean.TRUE);
        bzrProperties.setRevisioningCacheSize(new Integer(10));

        bzrProperties.setViewFormat("${root}/fisheye/viewrep/public${path}");
        bzrProperties.setChangeSetFormat("${root}/fisheye/changelog/public?cs=${rev}");
        bzrProperties.setFileAddedFormat("${root}/fisheye/viewrep/public${path}?r=${rev}");
        bzrProperties.setFileModifiedFormat("${root}/fisheye/viewrep/public${path}#${rev}");
        bzrProperties.setFileReplacedFormat("${root}/fisheye/viewrep/public${path}#${rev}");
        bzrProperties.setFileDeletedFormat("${root}/fisheye/viewrep/public${path}");

        bzrManager = new BazaarManagerImpl(1, propertySet);
        assertNull(bzrManager.getViewLinkFormat());

        bzrManager.update(bzrProperties);

        viewLinkFormat = bzrManager.getViewLinkFormat();
        /* Make sure the method returns a different ViewLinkFormat */
        assertNotNull(viewLinkFormat);

        assertEquals(bzrProperties.getViewFormat(), viewLinkFormat.getViewFormat());
        assertEquals(bzrProperties.getChangesetFormat(), viewLinkFormat.getChangesetFormat());
        assertEquals(bzrProperties.getFileAddedFormat(), viewLinkFormat.getFileAddedFormat());
        assertEquals(bzrProperties.getFileModifiedFormat(), viewLinkFormat.getFileModifiedFormat());
        assertEquals(bzrProperties.getFileReplacedFormat(), viewLinkFormat.getFileReplacedFormat());
        assertEquals(bzrProperties.getFileDeletedFormat(), viewLinkFormat.getFileDeletedFormat());

        /* Update the view link format */

        bzrProperties.setViewFormat("fakeViewFormat");
        bzrProperties.setChangeSetFormat("fakeChangesetFormat");
        bzrProperties.setFileAddedFormat("fakeFileAddedFormat");
        bzrProperties.setFileModifiedFormat("fakeFileModifiedFormat");
        bzrProperties.setFileReplacedFormat("fakeFileReplacedFormat");
        bzrProperties.setFileDeletedFormat("fakeFileDeletedFormat");

        bzrManager.update(bzrProperties);

        viewLinkFormat = bzrManager.getViewLinkFormat();
        /* Make sure the method returns a different ViewLinkFormat */
        assertNotNull(viewLinkFormat);

        assertEquals(bzrProperties.getViewFormat(), viewLinkFormat.getViewFormat());
        assertEquals(bzrProperties.getChangesetFormat(), viewLinkFormat.getChangesetFormat());
        assertEquals(bzrProperties.getFileAddedFormat(), viewLinkFormat.getFileAddedFormat());
        assertEquals(bzrProperties.getFileModifiedFormat(), viewLinkFormat.getFileModifiedFormat());
        assertEquals(bzrProperties.getFileReplacedFormat(), viewLinkFormat.getFileReplacedFormat());
        assertEquals(bzrProperties.getFileDeletedFormat(), viewLinkFormat.getFileDeletedFormat());
    }

    public void testResetWeblinksConfiguration() {
        bzrProperties.setRevisionIndexing(Boolean.TRUE);
        bzrProperties.setRevisioningCacheSize(new Integer(10));

        bzrProperties.setViewFormat("${root}/fisheye/viewrep/public${path}");
        bzrProperties.setChangeSetFormat("${root}/fisheye/changelog/public?cs=${rev}");
        bzrProperties.setFileAddedFormat("${root}/fisheye/viewrep/public${path}?r=${rev}");
        bzrProperties.setFileModifiedFormat("${root}/fisheye/viewrep/public${path}#${rev}");
        bzrProperties.setFileReplacedFormat("${root}/fisheye/viewrep/public${path}#${rev}");
        bzrProperties.setFileDeletedFormat("${root}/fisheye/viewrep/public${path}");

        bzrManager = new BazaarManagerImpl(1, propertySet);
        assertNull(bzrManager.getViewLinkFormat());

        bzrManager.update(bzrProperties);
        assertNotNull(bzrManager.getViewLinkFormat());

        /* Now, let's reset all the web links field */
        bzrProperties = new BazaarProperties();
        bzrProperties.setRevisionIndexing(Boolean.TRUE);
        bzrProperties.setRevisioningCacheSize(new Integer(10));

        bzrManager.update(bzrProperties);

        /* Make sure it is reset */
        assertNull(bzrManager.getViewLinkFormat());
    }
}

