package com.atlassian.jira.plugin.ext.bazaar.linkrenderer;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang.SystemUtils;
import org.vcs.bazaar.client.BazaarClientFactory;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.commandline.CommandLineLogMessage;
import org.vcs.bazaar.client.core.BazaarClientException;

import com.atlassian.jira.plugin.ext.bazaar.BazaarManagerImpl;
import com.atlassian.jira.plugin.ext.bazaar.MultipleBazaarRepositoryManager;
import com.opensymphony.module.propertyset.map.MapPropertySet;

public class TestLinkFormatRenderer extends TestCase {

    private Map subversionManagerProperties;

    private String bzrRepositoryUri;

    protected LinkFormatRenderer getLinkFormatRenderer() {
        return getLinkFormatRenderer(subversionManagerProperties);
    }

    protected LinkFormatRenderer getLinkFormatRenderer(final Map properties) {
        MapPropertySet propertySet;

        propertySet = new MapPropertySet();
        propertySet.setMap(properties);

        return new LinkFormatRenderer(new BazaarManagerImpl(1, propertySet));
    }

    protected void setUpLocalBzrRepository() throws BazaarClientException, URISyntaxException {
        File pathToLocalBzrRepository;
        
        BazaarClientFactory.setupBestAvailableBackend(true);

        pathToLocalBzrRepository = new File(new URI(bzrRepositoryUri));
        if (!pathToLocalBzrRepository.exists()){
//            BZRRepositoryFactory.createLocalRepository(
//                    pathToLocalBzrRepository,
//                    true,
//                    false);
        }
    }

    protected void tearDownLocalBzrRepository() throws IOException, URISyntaxException {
        File pathToLocalBzrRepository;

        pathToLocalBzrRepository = new File(new URI(bzrRepositoryUri));
        if (pathToLocalBzrRepository.exists())
            FileUtils.deleteDirectory(pathToLocalBzrRepository);
    }

    protected void setUp() throws Exception {
        File bzrRepository;

        super.setUp();

        do {
            bzrRepository = new File(SystemUtils.JAVA_IO_TMPDIR, "jira-bzr-plugin-local-bzr-repo" + RandomStringUtils.randomAlphanumeric(16));
        } while (bzrRepository.exists());

        bzrRepositoryUri = new StringBuffer("file://").append(bzrRepository.getAbsolutePath()).toString();

        subversionManagerProperties = new HashMap();
        subversionManagerProperties.put(MultipleBazaarRepositoryManager.BZR_BRANCH_ROOT_KEY, bzrRepositoryUri);
        subversionManagerProperties.put(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_PATH_KEY, "http://bzr.atlassian.com/fisheye/viewrep/public${path}");
        subversionManagerProperties.put(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_CHANGESET, "http://bzr.atlassian.com/fisheye/changelog/public?cs=${rev}");
        subversionManagerProperties.put(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_ADDED, "http://bzr.atlassian.com/fisheye/viewrep/public${path}?r=${rev}");
        subversionManagerProperties.put(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_MODIFIED, "http://bzr.atlassian.com/fisheye/viewrep/public${path}#${rev}");
        subversionManagerProperties.put(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_REPLACED, "http://bzr.atlassian.com/fisheye/viewrep/public${path}#${rev}");
        subversionManagerProperties.put(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_DELETED, "http://bzr.atlassian.com/fisheye/viewrep/public${path}");

        setUpLocalBzrRepository();
    }

    protected void tearDown() throws Exception {
        tearDownLocalBzrRepository();
        super.tearDown();
    }


    public void testGetRevisionLink() {
        IBazaarLogMessage revision;
        LinkFormatRenderer linkFormatRenderer;
        String link;
        Map properties;


        revision = new CommandLineLogMessage("1", "dchui", "branch_nick", new Date().toString(), "foo", null, null, null, null);
        linkFormatRenderer = getLinkFormatRenderer();
        link = linkFormatRenderer.getRevisionLink(revision);

        /* When a changeset format is defined */
        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/changelog/public?cs=1\">#1</a>", link);

        /* When a changeset format is not defined */
        properties = new HashMap(subversionManagerProperties);
        properties.remove(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_CHANGESET);

        linkFormatRenderer = getLinkFormatRenderer(properties);
        link = linkFormatRenderer.getRevisionLink(revision);
        assertEquals("#1", link);
    }

//    public void testGetCopySrcLink() {
//        Map _subversionManagerProperties;
//        LinkFormatRenderer linkFormatRenderer;
//        String link;
//        IBazaarLogMessage entry;
//        BZRLogEntryPath entryPathWithLeadingSlash;
//        BZRLogEntryPath entryPathWithoutLeadingSlash;
//
//
//        _subversionManagerProperties = new HashMap(this.subversionManagerProperties);
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        entry = new IBazaarLogMessage(null, 1, "dchui", new Date(), "foo");
//        entryPathWithLeadingSlash = new BZRLogEntryPath("/foo/bar", BazaarConstants.MODIFICATION, "/foo", -1);
//        entryPathWithoutLeadingSlash = new BZRLogEntryPath("/foo/bar", BazaarConstants.MODIFICATION, "foo", -1);
//
//        /* When path starts with a '/' */
//        link = linkFormatRenderer.getCopySrcLink(entry, entryPathWithLeadingSlash);
//        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/viewrep/public/foo\">/foo</a>", link);
//
//        /* When path does not start with a '/' */
//        link = linkFormatRenderer.getCopySrcLink(entry, entryPathWithoutLeadingSlash);
//        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/viewrep/public/foo\">/foo</a>", link);
//
//        /* When there is not path format and path starts with a '/' */
//        _subversionManagerProperties.remove(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_PATH_KEY);
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        link = linkFormatRenderer.getCopySrcLink(entry, entryPathWithLeadingSlash);
//        assertEquals("/foo", link);
//
//        /* When there is not path format and path does not start with a '/' */
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        link = linkFormatRenderer.getCopySrcLink(entry, entryPathWithoutLeadingSlash);
//        assertEquals("/foo", link);
//    }
//    
//    public void testGetChangeLinkPathOfFilesModified() {
//        Map _subversionManagerProperties;
//        LinkFormatRenderer linkFormatRenderer;
//        String link;
//        IBazaarLogMessage entry;
//        BZRLogEntryPath entryPathWithLeadingSlash;
//        BZRLogEntryPath entryPathWithoutLeadingSlash;
//
//
//        _subversionManagerProperties = new HashMap(this.subversionManagerProperties);
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        entry = new IBazaarLogMessage(null, 1, "dchui", new Date(), "foo");
//        entryPathWithLeadingSlash = new BZRLogEntryPath("/foo/bar", BazaarConstants.MODIFICATION, "/foo", -1);
//        entryPathWithoutLeadingSlash = new BZRLogEntryPath("/foo/bar", BazaarConstants.MODIFICATION, "foo", -1);
//
//        /* When path starts with a '/' */
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithLeadingSlash);
//        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/viewrep/public/foo/bar#1\">/foo/bar</a>", link);
//
//        /* When path does not start with a '/' */
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithoutLeadingSlash);
//        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/viewrep/public/foo/bar#1\">/foo/bar</a>", link);
//
//        /* When there is no path format and path starts with a '/' */
//        _subversionManagerProperties.remove(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_MODIFIED);
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithLeadingSlash);
//        assertEquals("/foo/bar", link);
//
//        /* When there is no path format and path does not start with a '/' */
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithoutLeadingSlash);
//        assertEquals("/foo/bar", link);
//    }
//
//    public void testGetChangeLinkPathOfFilesAdded() {
//        Map _subversionManagerProperties;
//        LinkFormatRenderer linkFormatRenderer;
//        String link;
//        IBazaarLogMessage entry;
//        BZRLogEntryPath entryPathWithLeadingSlash;
//        BZRLogEntryPath entryPathWithoutLeadingSlash;
//
//
//        _subversionManagerProperties = new HashMap(this.subversionManagerProperties);
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        entry = new IBazaarLogMessage(null, 1, "dchui", new Date(), "foo");
//        entryPathWithLeadingSlash = new BZRLogEntryPath("/foo/bar", BazaarConstants.ADDED, "/foo", -1);
//        entryPathWithoutLeadingSlash = new BZRLogEntryPath("/foo/bar", BazaarConstants.ADDED, "foo", -1);
//
//        /* When path starts with a '/' */
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithLeadingSlash);
//        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/viewrep/public/foo/bar?r=1\">/foo/bar</a>", link);
//
//        /* When path does not start with a '/' */
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithoutLeadingSlash);
//        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/viewrep/public/foo/bar?r=1\">/foo/bar</a>", link);
//
//        /* When there is no path format and path starts with a '/' */
//        _subversionManagerProperties.remove(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_ADDED);
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithLeadingSlash);
//        assertEquals("/foo/bar", link);
//
//        /* When there is no path format and path does not start with a '/' */
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithoutLeadingSlash);
//        assertEquals("/foo/bar", link);
//    }
//
//    public void testGetChangeLinkPathOfFilesReplaced() {
//        Map _subversionManagerProperties;
//        LinkFormatRenderer linkFormatRenderer;
//        String link;
//        IBazaarLogMessage entry;
//        BZRLogEntryPath entryPathWithLeadingSlash;
//        BZRLogEntryPath entryPathWithoutLeadingSlash;
//
//
//        _subversionManagerProperties = new HashMap(this.subversionManagerProperties);
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        entry = new IBazaarLogMessage(null, 1, "dchui", new Date(), "foo");
//        entryPathWithLeadingSlash = new BZRLogEntryPath("/foo/bar", BazaarConstants.REPLACED, "/foo", -1);
//        entryPathWithoutLeadingSlash = new BZRLogEntryPath("/foo/bar", BazaarConstants.REPLACED, "foo", -1);
//
//        /* When path starts with a '/' */
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithLeadingSlash);
//        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/viewrep/public/foo/bar#1\">/foo/bar</a>", link);
//
//        /* When path does not start with a '/' */
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithoutLeadingSlash);
//        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/viewrep/public/foo/bar#1\">/foo/bar</a>", link);
//
//        /* When there is no path format and path starts with a '/' */
//        _subversionManagerProperties.remove(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_REPLACED);
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithLeadingSlash);
//        assertEquals("/foo/bar", link);
//
//        /* When there is no path format and path does not start with a '/' */
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithoutLeadingSlash);
//        assertEquals("/foo/bar", link);
//    }
//
//    public void testGetChangeLinkPathOfFilesDeleted() {
//        Map _subversionManagerProperties;
//        LinkFormatRenderer linkFormatRenderer;
//        String link;
//        IBazaarLogMessage entry;
//        BZRLogEntryPath entryPathWithLeadingSlash;
//        BZRLogEntryPath entryPathWithoutLeadingSlash;
//
//
//        _subversionManagerProperties = new HashMap(this.subversionManagerProperties);
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        entry = new IBazaarLogMessage(null, 1, "dchui", new Date(), "foo");
//        entryPathWithLeadingSlash = new BZRLogEntryPath("/foo/bar", BazaarConstants.DELETED, "/foo", -1);
//        entryPathWithoutLeadingSlash = new BZRLogEntryPath("/foo/bar", BazaarConstants.DELETED, "foo", -1);
//
//        /* When path starts with a '/' */
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithLeadingSlash);
//        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/viewrep/public/foo/bar\">/foo/bar</a>", link);
//
//        /* When path does not start with a '/' */
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithoutLeadingSlash);
//        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/viewrep/public/foo/bar\">/foo/bar</a>", link);
//
//        /* When there is no path format and path starts with a '/' */
//        _subversionManagerProperties.remove(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_DELETED);
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithLeadingSlash);
//        assertEquals("/foo/bar", link);
//
//        /* When there is no path format and path does not start with a '/' */
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithoutLeadingSlash);
//        assertEquals("/foo/bar", link);
//    }
//
//    public void testGetChangeLinkPathOfFilesWithUnknownStatus() {
//        Map _subversionManagerProperties;
//        LinkFormatRenderer linkFormatRenderer;
//        String link;
//        IBazaarLogMessage entry;
//        BZRLogEntryPath entryPathWithLeadingSlash;
//        BZRLogEntryPath entryPathWithoutLeadingSlash;
//
//
//        _subversionManagerProperties = new HashMap(this.subversionManagerProperties);
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        entry = new IBazaarLogMessage(null, 1, "dchui", new Date(), "foo");
//        entryPathWithLeadingSlash = new BZRLogEntryPath("/foo/bar", 'X', "/foo", -1);
//        entryPathWithoutLeadingSlash = new BZRLogEntryPath("/foo/bar", 'X', "foo", -1);
//
//        /* When path starts with a '/' */
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithLeadingSlash);
//        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/viewrep/public/foo/bar#1\">/foo/bar</a>", link);
//
//        /* When path does not start with a '/' */
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithoutLeadingSlash);
//        assertEquals("<a href=\"http://bzr.atlassian.com/fisheye/viewrep/public/foo/bar#1\">/foo/bar</a>", link);
//
//        /* When there is no path format and path starts with a '/' */
//        _subversionManagerProperties.remove(MultipleBazaarRepositoryManager.BZR_LINKFORMAT_FILE_REPLACED);
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithLeadingSlash);
//        assertEquals("/foo/bar", link);
//
//        /* When there is no path format and path does not start with a '/' */
//        linkFormatRenderer = getLinkFormatRenderer(_subversionManagerProperties);
//        link = linkFormatRenderer.getChangePathLink(entry, entryPathWithoutLeadingSlash);
//        assertEquals("/foo/bar", link);
//    }
}
