package com.atlassian.jira.plugin.ext.bazaar.projecttabpanels;

import java.util.Collections;
import java.util.Date;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.jmock.Mock;
import org.jmock.cglib.MockObjectTestCase;
import org.vcs.bazaar.client.commandline.CommandLineLogMessage;

import webwork.action.ActionContext;

import com.atlassian.core.util.map.EasyMap;
import com.atlassian.jira.config.ConstantsManager;
import com.atlassian.jira.config.util.IndexPathManager;
import com.atlassian.jira.config.properties.ApplicationProperties;
import com.atlassian.jira.issue.IssueManager;
import com.atlassian.jira.issue.changehistory.ChangeHistoryManager;
import com.atlassian.jira.issue.search.SearchProvider;
import com.atlassian.jira.plugin.ext.bazaar.MultipleBazaarRepositoryManager;
import com.atlassian.jira.plugin.ext.bazaar.revisions.RevisionIndexer;
import com.atlassian.jira.plugin.projectpanel.ProjectTabPanelModuleDescriptor;
import com.atlassian.jira.project.Project;
import com.atlassian.jira.project.ProjectManager;
import com.atlassian.jira.project.browse.BrowseContext;
import com.atlassian.jira.project.version.Version;
import com.atlassian.jira.project.version.VersionManager;
import com.atlassian.jira.security.JiraAuthenticationContext;
import com.atlassian.jira.security.PermissionManager;
import com.atlassian.jira.security.Permissions;
import com.atlassian.jira.util.EasyList;
import com.atlassian.jira.web.action.ProjectActionSupport;
import com.atlassian.jira.web.bean.I18nBean;
import com.atlassian.plugin.PluginAccessor;
import com.opensymphony.user.User;

public class TestBazaarProjectTabPanel extends MockObjectTestCase {

    private Mock mockMultipleBazaarRepositoryManager;

    private Mock mockVersionManager;

    private Mock mockPermissionManager;

    private Mock mockRevisionIndexer;

    private Mock mockBrowseContext;

    private Mock mockProjectTabPanelModuleDescriptor;

    private Mock mockJiraAuthenticationContext;

    protected BazaarProjectTabPanel getBazaarProjectTabPanel() {
        BazaarProjectTabPanel bazaarProjectTabPanel;

        mockJiraAuthenticationContext = new Mock(JiraAuthenticationContext.class);
        mockProjectTabPanelModuleDescriptor.proxy();
        mockProjectTabPanelModuleDescriptor.expects(atMostOnce()).method("getHtml").with(eq("view"), isA(Map.class)).will(returnValue("foo"));


        bazaarProjectTabPanel= new BazaarProjectTabPanel(
		(JiraAuthenticationContext) mockJiraAuthenticationContext.proxy(),
                (MultipleBazaarRepositoryManager) mockMultipleBazaarRepositoryManager.proxy(),
                (VersionManager) mockVersionManager.proxy(),
                (PermissionManager) mockPermissionManager.proxy())
        {
            protected I18nBean getI18nBean(User user)
            {
                return null;
            }
        };

        bazaarProjectTabPanel.init((ProjectTabPanelModuleDescriptor) mockProjectTabPanelModuleDescriptor.proxy());
        return bazaarProjectTabPanel;
    }

    protected void setUp() throws Exception {
        // Mock mockMultipleBazaarRepositoryManager;
        MultipleBazaarRepositoryManager multipleBazaarRepositoryManager;

        super.setUp();

        mockMultipleBazaarRepositoryManager = new Mock(MultipleBazaarRepositoryManager.class);

        mockVersionManager = new Mock(VersionManager.class);
        mockVersionManager.expects(atMostOnce()).method("getVersionsReleased").withAnyArguments().will(returnValue(Collections.EMPTY_LIST));
        mockVersionManager.expects(atMostOnce()).method("getVersionsUnreleased").withAnyArguments().will(returnValue(Collections.EMPTY_LIST));

        mockPermissionManager = new Mock(PermissionManager.class);

        mockMultipleBazaarRepositoryManager = new Mock(MultipleBazaarRepositoryManager.class);
        mockMultipleBazaarRepositoryManager.expects(atLeastOnce()).method("getRepositoryList").withNoArguments().will(returnValue(Collections.EMPTY_LIST));
        multipleBazaarRepositoryManager = (MultipleBazaarRepositoryManager) mockMultipleBazaarRepositoryManager.proxy();

        mockRevisionIndexer = mock(
                        TestRevisionIndexer.class,
                        new Class[] {
                                MultipleBazaarRepositoryManager.class,
                                ApplicationProperties.class,
                                VersionManager.class,
                                IssueManager.class,
                                PermissionManager.class,
                                ChangeHistoryManager.class
                        },
                        new Object[] {
                                multipleBazaarRepositoryManager,
                                null,
                                null,
                                null,
                                mockPermissionManager.proxy(),
                                null
                        }
        );

        Class[] classArgs = new Class[]{ProjectManager.class, ConstantsManager.class, SearchProvider.class, PermissionManager.class, VersionManager.class, PluginAccessor.class};
        Object[] args = new Object[]{null, null, null, this.mockPermissionManager.proxy(), this.mockVersionManager.proxy(), null};
        mockBrowseContext = mock(BrowseContext.class, classArgs, args);

        mockProjectTabPanelModuleDescriptor = mock(ProjectTabPanelModuleDescriptor.class,
                new Class[] {JiraAuthenticationContext.class },
                new Object[] { null });

        this.mockMultipleBazaarRepositoryManager = new Mock(MultipleBazaarRepositoryManager.class);
    }

    public void testGetHtmlWithoutVersion() {
        Mock mockHttpServletRequest;
        Mock mockProject;
        HttpServletRequest httpServletRequest;
        BazaarProjectTabPanel bazaarProjectTabPanel;
        String html;

        mockProject = mock(Project.class);
        mockProject.reset();
        mockProject.expects(atLeastOnce()).method("getId").withNoArguments().will(returnValue(new Long(1)));
        mockProject.expects(once()).method("getKey").withNoArguments().will(returnValue("TP"));

        mockBrowseContext.reset();
        mockBrowseContext.expects(once()).method("getRemoteUser").withNoArguments().will(returnValue(null));
        mockBrowseContext.expects(once()).method("getSelectedProjectObject").withNoArguments().will(returnValue(mockProject.proxy()));

        mockHttpServletRequest = new Mock(HttpServletRequest.class);
        mockHttpServletRequest.expects(once()).method("getParameter").with(eq("selectedVersion")).will(returnValue(null));
        httpServletRequest = (HttpServletRequest) mockHttpServletRequest.proxy();
        ActionContext.setRequest(httpServletRequest);

        mockRevisionIndexer.reset();
        mockRevisionIndexer.expects(once()).method("getLogEntriesByProject").with(eq("TP"), NULL, eq(20)).will(
                returnValue(EasyMap.build(1L, EasyList.build(new CommandLineLogMessage("1", "dchui", "branch_nick", new Date().toString(), "foo TP", null, null, null, null))))
        );
        mockRevisionIndexer.expects(never()).method("getLogEntriesByVersion").withAnyArguments();

        mockMultipleBazaarRepositoryManager.reset();
        mockMultipleBazaarRepositoryManager.expects(once()).method("getRevisionIndexer").withNoArguments().will(returnValue(mockRevisionIndexer.proxy()));

        mockVersionManager.reset();
        mockVersionManager.expects(atLeastOnce()).method("getVersionsReleased").withAnyArguments().will(returnValue(Collections.EMPTY_LIST));
        mockVersionManager.expects(atLeastOnce()).method("getVersionsUnreleased").withAnyArguments().will(returnValue(Collections.EMPTY_LIST));

        bazaarProjectTabPanel = getBazaarProjectTabPanel();
        html = bazaarProjectTabPanel.getHtml((BrowseContext) mockBrowseContext.proxy());

        assertEquals("foo", html);
        mockRevisionIndexer.verify(); /* If this mock passes, we can be assured that the appropriate methods are called and the results are good */
    }

    public void testGetHtmlWithVersion() {
        Mock mockHttpServletRequest;
        Mock mockVersion;
        Mock mockProject;
        HttpServletRequest httpServletRequest;
        BazaarProjectTabPanel bazaarProjectTabPanel;
        String html;
        Version version;

        mockProject = mock(Project.class);
        mockProject.reset();
        mockProject.expects(atLeastOnce()).method("getId").withNoArguments().will(returnValue(new Long(1)));
        mockProject.expects(once()).method("getKey").withNoArguments().will(returnValue("TP"));

        mockBrowseContext.reset();
        mockBrowseContext.expects(once()).method("getRemoteUser").withNoArguments().will(returnValue(null));
        mockBrowseContext.expects(once()).method("getSelectedProjectObject").withNoArguments().will(returnValue(mockProject.proxy()));

        mockHttpServletRequest = new Mock(HttpServletRequest.class);
        mockHttpServletRequest.expects(once()).method("getParameter").with(eq("selectedVersion")).will(returnValue("1")); /* 1L as the version number */
        httpServletRequest = (HttpServletRequest) mockHttpServletRequest.proxy();
        ActionContext.setRequest(httpServletRequest);

        mockVersion = new Mock(Version.class);
        version = (Version) mockVersion.proxy();

        mockRevisionIndexer.reset();
        mockRevisionIndexer.expects(never()).method("getLogEntriesByProject").withAnyArguments();
        mockRevisionIndexer.expects(once()).method("getLogEntriesByVersion").with(same(version), NULL, eq(20)).will(returnValue(Collections.EMPTY_MAP));

        mockMultipleBazaarRepositoryManager.reset();
        mockMultipleBazaarRepositoryManager.expects(once()).method("getRevisionIndexer").withNoArguments().will(returnValue(mockRevisionIndexer.proxy()));

        mockVersionManager.reset();
        mockVersionManager.expects(atLeastOnce()).method("getVersionsReleased").withAnyArguments().will(returnValue(Collections.EMPTY_LIST));
        mockVersionManager.expects(atLeastOnce()).method("getVersionsUnreleased").withAnyArguments().will(returnValue(Collections.EMPTY_LIST));
        mockVersionManager.expects(once()).method("getVersion").with(eq(1L)).will(returnValue(version));

        bazaarProjectTabPanel = getBazaarProjectTabPanel();
        html = bazaarProjectTabPanel.getHtml((BrowseContext) mockBrowseContext.proxy());

        assertEquals("foo", html);
        mockRevisionIndexer.verify(); /* If this mock passes, we can be assured that the appropriate methods are called and the results are good */
        mockVersionManager.verify(); /* If this mock passes, we are assured the getVersion(Long):Version is called. That means version processing is good */
    }

    public void testShowPanelWhenMultipleBazaarManagerIsNotIndexing() {
        BazaarProjectTabPanel bazaarProjectTabPanel;
        ProjectActionSupport projectActionSupport;

        mockMultipleBazaarRepositoryManager.reset();
        mockMultipleBazaarRepositoryManager.expects(exactly(2)).method("isIndexingRevisions").withNoArguments().will(returnValue(false));

        mockPermissionManager.reset();
        mockPermissionManager.expects(never()).method("hasPermission").with(eq(Permissions.VIEW_VERSION_CONTROL), NULL, NULL);

        bazaarProjectTabPanel = getBazaarProjectTabPanel();
        projectActionSupport = new ProjectActionSupport(null, null) {
            public User getRemoteUser() {
                return null;
            }
        };

        assertFalse(bazaarProjectTabPanel.showPanel(projectActionSupport, null)); /* Not indexing and without permissions */
        assertFalse(bazaarProjectTabPanel.showPanel(projectActionSupport, null)); /* Not indexing but with permissions */
    }

    public void testShowPanelWhenMultipleBazaarManagerIsIndexing() {
        BazaarProjectTabPanel bazaarProjectTabPanel;
        ProjectActionSupport projectActionSupport;

        mockMultipleBazaarRepositoryManager.reset();
        mockMultipleBazaarRepositoryManager.expects(exactly(2)).method("isIndexingRevisions").withNoArguments().will(returnValue(true));

        mockPermissionManager.reset();
        mockPermissionManager.expects(exactly(2)).method("hasPermission").with(eq(Permissions.VIEW_VERSION_CONTROL), NULL, NULL).will(
                onConsecutiveCalls(
                        returnValue(false),
                        returnValue(true)
                )
        );

        bazaarProjectTabPanel = getBazaarProjectTabPanel();
        projectActionSupport = new ProjectActionSupport(null, null) {
            public User getRemoteUser() {
                return null;
            }
        };

        assertFalse(bazaarProjectTabPanel.showPanel(projectActionSupport, null)); /* Not indexing and without permissions */
        assertTrue(bazaarProjectTabPanel.showPanel(projectActionSupport, null)); /* Not indexing but with permissions */
    }

    protected static class TestRevisionIndexer extends RevisionIndexer {

        public TestRevisionIndexer(MultipleBazaarRepositoryManager multipleBazaarRepositoryManager, ApplicationProperties applicationProperties, VersionManager versionManager, IssueManager issueManager, PermissionManager permissionManager, ChangeHistoryManager changeHistoryManager, IndexPathManager indexPathManager) {
            super(multipleBazaarRepositoryManager, applicationProperties, versionManager, issueManager, permissionManager, changeHistoryManager, indexPathManager);
        }
    }
}
